###########################################################################
## Filename:      start.S
## Version:       $Id: start.S,v 1.18 2000/07/14 17:25:11 erikm Exp $
## Copyright:     Copyright (C) 1999, Erik Mouw
## Author:        Erik Mouw <J.A.K.Mouw@its.tudelft.nl>
## Description:   blob start code
## Created at:    Sun Jul 18 20:29:08 1999
## Modified by:   Erik Mouw <J.A.K.Mouw@its.tudelft.nl>
## Modified at:   Fri Jul  14 17:55:36 2000
###########################################################################
/*
 * start.S: blob start code
 *
 * Copyright (C) 1999  Erik Mouw (J.A.K.Mouw@its.tudelft.nl) and 
 *                     Jan-Derk Bakker (J.D.Bakker@its.tudelft.nl)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */
/*
 * This is the blob start code. The SA-1100 jumps to address 0x00000000
 * after a reset. We put a single branch code at this position which jumps
 * to a safe region to do the actual setup. All other vectors just point
 * to an endless loop for the moment.
 *
 * Documentation: 
 * [1] Intel Corporation, "Intel StrongARM SA-1100 Microprocessor
 *     Developer's Manual", April 1999
 * [2] S. Furber, "ARM System Architecture", Addison Wesley Longman
 *     Ltd., Essex, England, 1996.
 * [3] Intel Corporation, "Intel StrongARM SA-1110 Microprocessor
 *     Advanced Developer's manual, December 1999
 */

.ident "$Id: start.S,v 1.18 2000/07/14 17:25:11 erikm Exp $"

//Uncomment next line for 16-bit boot, comment for 32-bit boot
//#define EPROM

#ifdef HAVE_CONFIG_H
# include "config.h"
#endif
#include "sa1111.h"
.text

	
	
	
/* Jump vector table as in table 3.1 in [1] */
.globl _start
_start:	b	reset
	b	undefined_instruction
	b	software_interrupt
	b	abort_prefetch
	b	abort_data
	b	not_used
	b	irq
	b	fiq

	
	
	
reset:
	/* Register addresses can be found in [1] Appendix A */

	/* First, mask **ALL** interrupts */
	mov	r1, #0x90000000
	add	r1, r1, #0x50000
	mov	r2, #0x00
	str	r2, [r1, #0x4]

	
	/* Switch the CPU to 221 MHz by writing the PPCR. */
	/* Don't worry, 221 MHz is also safe for 190 MHz CPUs. */
	/* 0x05 is 128.9 Mhz. 0x06  is 143.2, 07 is 157.5 for 3.5795 M clk*/
	/* 0x0A is 200.5 */
	mov	r1, #0x90000000
	add	r1, r1, #0x20000
	mov	r2, #0x05
	str	r2, [r1, #0x14]


	/* Setup memory */
	

	/* This part is actually for the Assabet only. If your board
	 * uses other settings, you'll have to ifdef them here.
	 */
	/* Set up the SDRAM */
	mov	r1, #0xA0000000		/* MDCNFG base address */

	ldr	r2, =0xAAAAAA7F
	str	r2, [r1, #0x04]		/* MDCAS00 */
	str	r2, [r1, #0x20]		/* MDCAS20 */

	ldr	r2, =0xAAAAAAAA
	str	r2, [r1, #0x08]		/* MDCAS01 */
	str	r2, [r1, #0x24]		/* MDCAS21 */

	ldr	r2, =0xAAAAAAAA
	str	r2, [r1, #0x0C]		/* MDCAS02 */
	str	r2, [r1, #0x28]		/* MDCAS22 */

	ldr	r2, =0x007000E1		/* MDREFR */
	str	r2, [r1, #0x1C]
	
	ldr	r2, =0x0006154		/* MDCNFG */
	str	r2, [r1, #0x00]

	/* Issue read requests to disabled bank to start refresh */

	ldr	r1, =0xC0000000

.rept	8	
	ldr	r0, [r1]
.endr

	mov	r1, #0xA0000000		/* MDCNFG base address */
	


	ldr	r2, =0x0006157		/*  banks to be enabled */ 
	str	r2, [r1, #0x00]		/* MDCNFG */
	
/* Static memory chip selects on Assabet: */

#ifdef EPROM
	ldr	r2, =0xfff8fffc			/* MCS0 */
#else
       	ldr	r2, =0xfffcfff8			/* MCS0 */
#endif
	 
	str	r2, [r1, #0x10]

	ldr	r2, =0x22212419			/* MCS1 */
	str	r2, [r1, #0x14]
/* lower nibble of MCS2  controls SA1111 chip select nCS4 */
	
	ldr	r2, =0x42196669			/* MCS2 */
	str	r2, [r1, #0x2C]

	ldr	r2, =0xafccafcc			/* SMCNFG */
	str	r2, [r1, #0x30]

/* Set up PCMCIA space */

/*
	ldr	r2, =0x994a994a
	str	r2, [r1, #0x18]
*/

/* All SDRAM memory settings should be ready to go... */
/* For best performance, should fill out remaining memory config regs: */

# ----------------------- Start of GPIO code --------------------
/* Vivek */

	ldr	r1,=0x90040000
	ldr	r4,=0x061C01D8	/* O/P GP3,4,6,7,8,18,19,20,25,26	*/
				/* I/P GP2,5,10,11,12,13,14,15	  	*/
	str	r4,[r1,#0x4]	/* GPDR */

#	ldr	r4,=0x08600000	/* Alternate functions for pins 21,22,27*/
#	str	r4,[r1,#0x1C]	/* GAFR	*/

#	ldr	r4,=0xFFFFFFFF	/* Clear all GPIO bits			*/
#	str	r4,[r1,#0xC]	/* GPCR					*/

	ldr	r4,=0x04000000	/* Bit 26 (LCD) is set			*/
	str	r4,[r1,#0x8]	/* GPSR	*/

# ------------------------ end of GPIO code ---------------------

# ------------------------- set palette in DRAM  ---------------------
	ldr	r1,=0xC8000000		/* Palette start in DRAM	  */
	mov	r2,#0x0			/* Palette entry		  */

palette_loop:
	strh	r2,[r1],#0x02		/* Store Palette Entry, post inc  */
	add	r2,r2,#0x1		/* Increment palette value	  */
	cmp	r2,#0x10
	bne	palette_loop

# ----------------------- end set palette in DRAM  -------------------

# ---------------------- transfer logo flash->DRAM  ------------------

	ldr	r4,=0x10000		/* Source flash LOGO_START(main.h) */
	mov	r2,#0x0			/* Offset from source		   */
	ldr	r3,=0x9600 		/* Size of image 		   */
	ldr	r1,=0xC8000020		/* Destination in DRAM	(2nd bank) */

loop:
	ldr	r5,[r4],#0x04		/* Read from source,post increment  */
	str	r5,[r1],#0x04		/* Write to DRAM,post increment	  */
	subs 	r3,r3,#0x1		/* Decrement size		  */
	bne	loop
	
# -------------------- transfer logo flash->DRAM done ----------------

# ------------------------- lcd code begins here ---------------------

	ldr	r1, =0xB0100000
	ldr	r4, =0x08012130
        str     r4,[r1,#0x20]		@lcd_con_1

	ldr	r4, =0x000004EF
        str     r4,[r1,#0x24]		@lcd_con_2

	ldr	r4,=0x0000640A		@ 640C at 162 MHz, 
					@ 6410 at 206 MHz, 640a at 133 MHz
        str     r4,[r1,#0x28]		@lcd_con_3

	/* Vivek - Enabling LCD DMA for startup logo */
	mov	r4,#0xC8000000		@ See LOGO_START in main.h
	str	r4,[r1,#0x10]

	mov	r4,#0x03       		@lcd_con_0=0x03 LEN has been done
        str     r4,[r1,#0]
# ------------------------- end of lcd code ---------------------

# -------------------- Start of more GPIO code ------------------

	ldr	r1,=0x90040000
	ldr	r4,=0x06000000	/* Bit 25 & 26 (LCD) are set		*/
	str	r4,[r1,#0x8]	/* GPSR	*/

# --------------------- end of more GPIO code -------------------

	/* Try to transmit over the serial port III. */
	/* Do port setup the paranoid way */

	/* Now clear all 'sticky' bits in serial registers, cf. [1] 11.11 */

	/* Serial port 1 on Assabet */
	ldr r0,=0x80020060 	/* GPCLK/UART select,select UART */
	mov r2,#1
	str r2,[r0]

	/* Now clear all 'sticky' bits in serial I registers, cf. [1] 11.11 */
	mov	r1, #0x80000000
	add	r1, r1, #0x10000
	

	mov	r2, #0xFF
	str	r2, [r1, #0x1C]
	
	/* Set the serial port to sensible defaults: no break, no interrupts, */
	/* no parity, 8 databits, 1 stopbit. */
	mov	r2, #0x00
	str	r2, [r1, #0x0C]
	mov	r2, #0x08
	str	r2, [r1, #0x00]

	/* Set BRD to 5, for a baudrate of 38k4 ([1] 11.11.4.1) */
	/* Set BRD to 23, for a baudrate of 9k6 ([1] 11.11.4.1) */
	mov	r2, #0x00
	str	r2, [r1, #0x04]
	mov	r2, #23
	str	r2, [r1, #0x08]


 /* Prepare to send some characters out in a loop */
	mov	r1, #0x80000000

	add	r1, r1, #0x10000

	/* Enable the transmitter */
	mov	r2, #0x02
	str	r2, [r1, #0x0C]

	/* Send out a welcome message */
	adr	r0, welcome
	bl	print_str

#if 0 /* Vivek */

test_mem:
	/* Start the memory tester, hardcoded bank 0 & 1 for now */
	adr	r0, start_test
	bl	print_str

	/* Erase memory */
	adr	r0, zeroing_mem
	bl	print_str

	mov	r4, #0xC0000000
	mov	r5, #0xD0000000
	mov	r6, #0x400000
	sub	r6, r6, #0x01

	mov	r0, r4
	bl	print_hex

zero_loop:
	mov	r0, #0x0
	mov	r1, #0x0
	mov	r2, #0x0
	mov	r3, #0x0
	mov	r7, #0x0
	mov	r8, #0x0
	mov	r9, #0x0
	mov	r10, #0x0
zero_loop1:
	stmia	r4!, {r0-r3, r7-r10}
	stmia	r4!, {r0-r3, r7-r10}
	stmia	r4!, {r0-r3, r7-r10}
	stmia	r4!, {r0-r3, r7-r10}
	tst	r4, r6
	bne	zero_loop1
	adr	r0, del10
	bl	print_str
	mov	r0, r4
	bl	print_hex
	cmp	r4, r5
	blt	zero_loop

	adr	r0, zeroing_done
	bl	print_str

	mov	r4, #0xC0000000
	mov	r5, #0xD0000000
	mov	r6, r4 /* Base for current block */
	mov	r7, #0x0 /* Diff between r4 and [r4] */

	/* Go hunting for aliases */
alias_loop:
	ldr	r8, [r4]
	cmp	r8, #0x0
	streq	r4, [r4] /* If the value was zero, load r4 to it */
	moveq	r8, r4
 	sub	r9, r4, r8
	cmp	r9, r7 /* Is this a new block ? */
	bne	alias_detected
	add	r4, r4, #0x1000
	cmp	r4, r5
	blt	alias_loop	
	b	alias_done

alias_detected:

#	adr	r0, block_at_str
#	bl	print_str
#	mov	r0, r6
#	bl	print_hex
#	adr	r0, length_str
#	bl	print_str
#	sub	r0, r4, r6
#	bl	print_hex
	sub	r8, r6, r7
	cmp	r7, #0x0	/* Is this a 'new' block ? */
	moveq	r10, r6		/* If so, remember the start of this block...*/ 
	moveq	r11, r4		/* ...and the start of the next block */
	mov	r6, r4
	mov	r7, r9
	beq	alias_loop
#	adr	r0, alias_for_str
#	bl	print_str
#	mov	r0, r8
#	bl	print_hex
	b	alias_loop

alias_done:
#else
/* Dirty HACK for now.... Vivek */
	ldr	r11,=0xC9000000
	ldr	r10,=0xC8000000
#endif

	/* Set up the stack pointer */
	sub	sp, r11, #0x04

	/* Arg 0 of the C code is the start of the block it can use@ */
	/* arg 1 is the size of that block. */
	mov	r0, r10
	sub	r1, r11, r10

	/* Jump to the C code */
jump_to_c:
	bl	c_main

	/* The c code should never return ! */
	b	reset


blinky:
	/* The old blinker */
	mov     r2, #0x90000000
	add     r2, r2, #0x40000
	mov     r1, #0x00800000

old_led_on:
	/* turn on the LED by writing the GPSR (GPIO Pin output Set
	Register) */
	str	r1, [r2, #0x08]
	mov	r4, #0x10000
loop1:
	subs	r4, r4, #1
	bne	loop1

old_led_off:
	/* turn off the LED by writing the GPCR (GPIO Pin output Clear
	Register) */
	str	r1, [r2, #0x0c]

	mov     r4, #0x10000
loop2:
	subs    r4, r4, #1
	bne	loop2

	/* and loop forever */
	b	old_led_on

.align 4
sa1111_string:
	.string "testing sa111"
	
.align 4
welcome:
	.string "\n\r\n\r This is the Simputer!\n\r"

.align 4
int_flash_str:
	.string "Running from internal Flash.\n\r"

.align 4
start_test:
	.string "Starting the memory tester...\n\r"

.align 4
zeroing_mem:
	.string "Zeroing memory..."

.align 4
zeroing_done:
	.string "\n\rZeroing done. Testing for aliases...\n\r"

.align 4
del10:
	.string "\b\b\b\b\b\b\b\b\b\b"

.align 4
block_at_str:
	.string "\n\rBlock at "

.align 4
length_str:
	.string " with length "

.align 4
alias_for_str:
	.string " is an alias of "
.align 4

	/* Subroutine that sends a string over the serial port */
	/* The address of the string should be in r0 */
print_str:
	/* Save the return address */
	mov	r13, r14
	mov	r2, r0
prs1:
	ldrsb	r0, [r2]
	add	r2, r2, #0x01
	ands	r0, r0, #0xFF
	beq	prs2
	bl	print_byte
	b	prs1

prs2:
	/* Return */
	mov	pc, r13


	/* Subroutine to send a hex word (in r0) over the serial port */
print_hex:
	mov	r13, r14
	mov	r2, r0
	mov	r3, #0x08
	mov	r0, #0x30
	bl	print_byte
	mov	r0, #0x78
	bl	print_byte
prh1:
	and	r0, r2, #0xF0000000
	mov	r0, r0, lsr #28
	add	r0, r0, #0x30
	cmp	r0, #0x3A
	addge	r0, r0, #0x07
	bl	print_byte
	mov	r2, r2, lsl #4
	subs	r3, r3, #0x01	
	bne	prh1

	mov	pc, r13

	/* Subroutine that sends a byte over the serial port. */
	/* The byte is in r0 */
print_byte:
	/* Wait for room in the tx fifo */
	mov     r1, #0x80000000
	add	r1, r1, #0x10000
	ldr     r1, [r1, #0x1C]
	ands    r1, r1, #0x01
	beq	print_byte

	mov     r1, #0x80000000

	add     r1, r1, #0x10000

	str	r0, [r1, #0x14]
	mov	pc, r14

	


.globl	data_to_flash
.globl	data_from_flash
	/* Subroutine that takes data in r0 and formats it so it will be in */
	/* the correct order for the internal flash */
data_to_flash:
	mov	r1, #0x0
	
	tst	r0, #0x00000001
	orrne	r1, r1, #0x00001000
	tst	r0, #0x00000002
	orrne	r1, r1, #0x00004000
	tst	r0, #0x00000004
	orrne	r1, r1, #0x00000800
	tst	r0, #0x00000008
	orrne	r1, r1, #0x00000200
	tst	r0, #0x00000010
	orrne	r1, r1, #0x00000001
	tst	r0, #0x00000020
	orrne	r1, r1, #0x00000004
	tst	r0, #0x00000040
	orrne	r1, r1, #0x00000080
	tst	r0, #0x00000080
	orrne	r1, r1, #0x00000020

	tst	r0, #0x00000100
	orrne	r1, r1, #0x00002000
	tst	r0, #0x00000200
	orrne	r1, r1, #0x00008000
	tst	r0, #0x00000400
	orrne	r1, r1, #0x00000400
	tst	r0, #0x00000800
	orrne	r1, r1, #0x00000100
	tst	r0, #0x00001000
	orrne	r1, r1, #0x00000002
	tst	r0, #0x00002000
	orrne	r1, r1, #0x00000008
	tst	r0, #0x00004000
	orrne	r1, r1, #0x00000040
	tst	r0, #0x00008000
	orrne	r1, r1, #0x00000010

	tst	r0, #0x00010000
	orrne	r1, r1, #0x00100000
	tst	r0, #0x00020000
	orrne	r1, r1, #0x00400000
	tst	r0, #0x00040000
	orrne	r1, r1, #0x00080000
	tst	r0, #0x00080000
	orrne	r1, r1, #0x00020000
	tst	r0, #0x00100000
	orrne	r1, r1, #0x01000000
	tst	r0, #0x00200000
	orrne	r1, r1, #0x04000000
	tst	r0, #0x00400000
	orrne	r1, r1, #0x80000000
	tst	r0, #0x00800000
	orrne	r1, r1, #0x20000000

	tst	r0, #0x01000000
	orrne	r1, r1, #0x00200000
	tst	r0, #0x02000000
	orrne	r1, r1, #0x00800000
	tst	r0, #0x04000000
	orrne	r1, r1, #0x00040000
	tst	r0, #0x08000000
	orrne	r1, r1, #0x00010000
	tst	r0, #0x10000000
	orrne	r1, r1, #0x02000000
	tst	r0, #0x20000000
	orrne	r1, r1, #0x08000000
	tst	r0, #0x40000000
	orrne	r1, r1, #0x40000000
	tst	r0, #0x80000000
	orrne	r1, r1, #0x10000000

	mov	r0, r1
	mov	pc, r14

	/* Takes data received from the flash, and unshuffles it. */
data_from_flash:
	mov	r1, #0x00

	tst	r0, #0x00000001
	orrne	r1, r1, #0x00000010
	tst	r0, #0x00000002
	orrne	r1, r1, #0x00001000
	tst	r0, #0x00000004
	orrne	r1, r1, #0x00000020
	tst	r0, #0x00000008
	orrne	r1, r1, #0x00002000
	tst	r0, #0x00000010
	orrne	r1, r1, #0x00008000
	tst	r0, #0x00000020
	orrne	r1, r1, #0x00000080
	tst	r0, #0x00000040
	orrne	r1, r1, #0x00004000
	tst	r0, #0x00000080
	orrne	r1, r1, #0x00000040

	tst	r0, #0x00000100
	orrne	r1, r1, #0x00000800
	tst	r0, #0x00000200
	orrne	r1, r1, #0x00000008
	tst	r0, #0x00000400
	orrne	r1, r1, #0x00000400
	tst	r0, #0x00000800
	orrne	r1, r1, #0x00000004
	tst	r0, #0x00001000
	orrne	r1, r1, #0x00000001
	tst	r0, #0x00002000
	orrne	r1, r1, #0x00000100
	tst	r0, #0x00004000
	orrne	r1, r1, #0x00000002
	tst	r0, #0x00008000
	orrne	r1, r1, #0x00000200

	tst	r0, #0x00010000
	orrne	r1, r1, #0x08000000
	tst	r0, #0x00020000
	orrne	r1, r1, #0x00080000
	tst	r0, #0x00040000
	orrne	r1, r1, #0x04000000
	tst	r0, #0x00080000
	orrne	r1, r1, #0x00040000
	tst	r0, #0x00100000
	orrne	r1, r1, #0x00010000
	tst	r0, #0x00200000
	orrne	r1, r1, #0x01000000
	tst	r0, #0x00400000
	orrne	r1, r1, #0x00020000
	tst	r0, #0x00800000
	orrne	r1, r1, #0x02000000

	tst	r0, #0x01000000
	orrne	r1, r1, #0x00100000
	tst	r0, #0x02000000
	orrne	r1, r1, #0x10000000
	tst	r0, #0x04000000
	orrne	r1, r1, #0x00200000
	tst	r0, #0x08000000
	orrne	r1, r1, #0x20000000
	tst	r0, #0x10000000
	orrne	r1, r1, #0x80000000
	tst	r0, #0x20000000
	orrne	r1, r1, #0x00800000
	tst	r0, #0x40000000
	orrne	r1, r1, #0x40000000
	tst	r0, #0x80000000
	orrne	r1, r1, #0x00400000

	mov	r0, r1
	mov	pc, r14
	




#if defined USE_SA1110
tenthSecDelay:  /* Well, actually not that much :) */
	mov	r3, #0x90000000
	add	r3, r3, #0x10000
	ldr	r4, [r3, #0x4]
1:	ldr	r6, [r3, #0x4]
	sub	r6, r6, r4
	cmp	r6, #0xc0
	ble	1b
	mov	pc, lr
#endif




undefined_instruction:
	b	undefined_instruction



	
software_interrupt:
	b	software_interrupt




abort_prefetch:
	b	abort_prefetch




abort_data:
	b	abort_data




not_used:
	b	not_used



	
irq:
	b	irq




fiq:
	b	fiq

















